@@ -1,5 +1,6 @@
 """Package for dataprofiler."""
 from . import settings
+from ._version import get_versions
 from .data_readers.data import Data
 from .dp_logging import get_logger, set_verbosity
 from .labelers.data_labelers import (
@@ -8,6 +9,7 @@ from .labelers.data_labelers import (
     UnstructuredDataLabeler,
     train_structured_labeler,
 )
+from .plugins import load_plugins
 from .profilers.graph_profiler import GraphProfiler
 from .profilers.profile_builder import (
     Profiler,
@@ -17,23 +19,8 @@ from .profilers.profile_builder import (
 from .profilers.profiler_options import ProfilerOptions
 from .reports import graphs
 from .validators.base_validators import Validator
-from .version import __version__
 
-try:
-    import snappy
-except ImportError:
-    import warnings
-
-    warnings.warn(
-        "Snappy must be installed to use parquet/avro datasets."
-        "\n\n"
-        "For macOS use Homebrew:\n"
-        "\t`brew install snappy`"
-        "\n\n"
-        "For linux use apt-get:\n`"
-        "\tsudo apt-get -y install libsnappy-dev`\n",
-        ImportWarning,
-    )
+__version__ = get_versions()["version"]
 
 
 def set_seed(seed=None):
@@ -41,3 +28,6 @@ def set_seed(seed=None):
     if seed is not None and (not isinstance(seed, int) or seed < 0):
         raise ValueError("Seed should be a non-negative integer.")
     settings._seed = seed
+
+
+load_plugins()
