@@ -1,4 +1,4 @@
-"""Contains class that saves and loads speradsheet data."""
+"""Contains class that saves and loads spreadsheet data."""
 import csv
 import random
 import re
@@ -87,6 +87,7 @@ class CSVData(SpreadSheetDataMixin, BaseData):
         self._checked_header: bool = "header" in options and self._header != "auto"
         self._default_delimiter: str = ","
         self._default_quotechar: str = '"'
+        self._sample_nrows: Optional[int] = options.get("sample_nrows", None)
 
         if data is not None:
             self._load_data(data)
@@ -115,6 +116,11 @@ class CSVData(SpreadSheetDataMixin, BaseData):
         """Return header."""
         return self._header
 
+    @property
+    def sample_nrows(self) -> Optional[int]:
+        """Return sample_nrows."""
+        return self._sample_nrows
+
     @property
     def is_structured(self) -> bool:
         """Determine compatibility with StructuredProfiler."""
@@ -168,6 +174,10 @@ class CSVData(SpreadSheetDataMixin, BaseData):
                 raise ValueError(
                     "'record_samples_per_line' must be an int " "more than 0"
                 )
+        if "sample_nrows" in options:
+            value = options["sample_nrows"]
+            if not isinstance(value, int) or value < 0:
+                raise ValueError("'sample_nrows' must be an int more than 0")
         return options
 
     @staticmethod
@@ -549,6 +559,7 @@ class CSVData(SpreadSheetDataMixin, BaseData):
             data_buffered,
             self.delimiter,
             cast(Optional[int], self.header),
+            self.sample_nrows,
             self.selected_columns,
             read_in_string=True,
         )
@@ -569,11 +580,8 @@ class CSVData(SpreadSheetDataMixin, BaseData):
                 self._quotechar = quotechar
 
             if self._header == "auto":
-                self._header = cast(
-                    int,
-                    self._guess_header_row(
-                        data_as_str, self._delimiter, self._quotechar
-                    ),
+                self._header = self._guess_header_row(
+                    data_as_str, self._delimiter, self._quotechar
                 )
                 self._checked_header = True
 
@@ -598,9 +606,10 @@ class CSVData(SpreadSheetDataMixin, BaseData):
             input_file_path,
             self.delimiter,
             cast(Optional[int], self.header),
+            self.sample_nrows,
             self.selected_columns,
             read_in_string=True,
-            encoding=cast(str, self.file_encoding),
+            encoding=self.file_encoding,
         )
 
     def _get_data_as_records(self, data: pd.DataFrame) -> List[str]:
@@ -758,4 +767,4 @@ class CSVData(SpreadSheetDataMixin, BaseData):
             header=self.header, delimiter=self.delimiter, quotechar=self.quotechar
         )
         super().reload(input_file_path, data, options)
-        self.__init__(self.input_file_path, data, options)  # type: ignore
+        CSVData.__init__(self, self.input_file_path, data, options)
