@@ -1,9 +1,14 @@
 """Contains functions for data readers."""
 import json
+import logging
+import os
+import random
 import re
 import urllib
 from collections import OrderedDict
 from io import BytesIO, StringIO, TextIOWrapper
+from itertools import islice
+from math import floor, log, log1p
 from typing import (
     Any,
     Dict,
@@ -17,14 +22,17 @@ from typing import (
     cast,
 )
 
+import boto3
+import botocore
 import dateutil
+import numpy as np
 import pandas as pd
 import pyarrow.parquet as pq
 import requests
 from chardet.universaldetector import UniversalDetector
 from typing_extensions import TypeGuard
 
-from .. import dp_logging
+from .. import dp_logging, rng_utils
 from .._typing import JSONType, Url
 from .filepath_or_buffer import FileOrBufferHandler, is_stream_buffer  # NOQA
 
@@ -268,10 +276,102 @@ def read_json(
     return lines
 
 
+def reservoir(file: TextIOWrapper, sample_nrows: int) -> list:
+    """
+    Implement the mathematical logic of Reservoir sampling.
+
+    :param file: wrapper of the opened csv file
+    :type file: TextIOWrapper
+    :param sample_nrows: number of rows to sample
+    :type sample_nrows: int
+
+    :raises: ValueError()
+
+    :return: sampled values
+    :rtype: list
+    """
+    # Copyright 2021 Oscar Benjamin
+    #
+    # Permission is hereby granted, free of charge, to any person obtaining a copy
+    # of this software and associated documentation files (the "Software"), to deal
+    # in the Software without restriction, including without limitation the rights
+    # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+    # copies of the Software, and to permit persons to whom the Software is
+    # furnished to do so, subject to the following conditions:
+    #
+    # The above copyright notice and this permission notice shall be included in
+    # all copies or substantial portions of the Software.
+    #
+    # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+    # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+    # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+    # AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+    # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+    # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+    # SOFTWARE.
+    # https://gist.github.com/oscarbenjamin/4c1b977181f34414a425f68589e895d1
+
+    iterator = iter(file)
+    values = list(islice(iterator, sample_nrows))
+
+    irange = range(len(values))
+    indices = dict(zip(irange, irange))
+
+    kinv = 1 / sample_nrows
+    W = 1.0
+    rng = rng_utils.get_random_number_generator()
+
+    while True:
+        W *= rng.random() ** kinv
+        # random() < 1.0 but random() ** kinv might not be
+        # W == 1.0 implies "infinite" skips
+        if W == 1.0:
+            break
+        # skip is geometrically distributed with parameter W
+        skip = floor(log(rng.random()) / log1p(-W))
+        try:
+            newval = next(islice(iterator, skip, skip + 1))
+        except StopIteration:
+            break
+        # Append new, replace old with dummy, and keep track of order
+        remove_index = rng.integers(0, sample_nrows)
+        values[indices[remove_index]] = str(None)
+        indices[remove_index] = len(values)
+        values.append(newval)
+
+    values = [values[indices[i]] for i in irange]
+    return values
+
+
+def rsample(file_path: TextIOWrapper, sample_nrows: int, args: dict) -> StringIO:
+    """
+    Implement Reservoir Sampling to sample n rows out of a total of M rows.
+
+    :param file_path: path of the csv file to be read in
+    :type file_path: TextIOWrapper
+    :param sample_nrows: number of rows being sampled
+    :type sample_nrows: int
+    :param args: options to read the csv file
+    :type args: dict
+    """
+    header = args["header"]
+    result = []
+
+    if header is not None:
+        result = [[next(file_path) for i in range(header + 1)][-1]]
+        args["header"] = 0
+
+    result += reservoir(file_path, sample_nrows)
+
+    fo = StringIO("".join([i if (i[-1] == "\n") else i + "\n" for i in result]))
+    return fo
+
+
 def read_csv_df(
     file_path: Union[str, BytesIO, TextIOWrapper],
     delimiter: Optional[str],
     header: Optional[int],
+    sample_nrows: Optional[int] = None,
     selected_columns: List[str] = [],
     read_in_string: bool = False,
     encoding: Optional[str] = "utf-8",
@@ -314,26 +414,113 @@ def read_csv_df(
 
     # account for py3.6 requirement for pandas, can remove if >= py3.7
     is_buf_wrapped = False
+    is_file_open = False
     if isinstance(file_path, BytesIO):
         # a BytesIO stream has to be wrapped in order to properly be detached
         # in 3.6 this avoids read_csv wrapping the stream and closing too early
         file_path = TextIOWrapper(file_path, encoding=encoding)
         is_buf_wrapped = True
+    elif isinstance(file_path, str):
+        file_path = open(file_path, encoding=encoding)
+        is_file_open = True
 
-    fo = pd.read_csv(file_path, **args)
+    file_data = file_path
+    if sample_nrows:
+        file_data = rsample(file_path, sample_nrows, args)
+    fo = pd.read_csv(file_data, **args)
     data = fo.read()
 
     # if the buffer was wrapped, detach it before returning
     if is_buf_wrapped:
         file_path = cast(TextIOWrapper, file_path)
         file_path.detach()
+    elif is_file_open:
+        file_path.close()
     fo.close()
 
     return data
 
 
+def convert_unicode_col_to_utf8(input_df: pd.DataFrame) -> pd.DataFrame:
+    """
+    Convert all unicode columns in input dataframe to utf-8.
+
+    :param input_df: input dataframe
+    :type input_df: pd.DataFrame
+    :return: corrected dataframe
+    :rtype: pd.DataFrame
+    """
+    # Convert all the unicode columns to utf-8
+    input_column_types = input_df.apply(
+        lambda x: pd.api.types.infer_dtype(x.values, skipna=True)
+    )
+
+    mixed_and_unicode_cols = input_column_types[
+        input_column_types == "unicode"
+    ].index.union(input_column_types[input_column_types == "mixed"].index)
+
+    for iter_column in mixed_and_unicode_cols:
+        # Encode sting to bytes
+        input_df[iter_column] = input_df[iter_column].apply(
+            lambda x: x.encode("utf-8").strip() if isinstance(x, str) else x
+        )
+
+        # Decode bytes back to string
+        input_df[iter_column] = input_df[iter_column].apply(
+            lambda x: x.decode("utf-8").strip() if isinstance(x, bytes) else x
+        )
+
+    return input_df
+
+
+def sample_parquet(
+    file_path: str,
+    sample_nrows: int,
+    selected_columns: Optional[List[str]] = None,
+    read_in_string: bool = False,
+) -> Tuple[pd.DataFrame, pd.Series]:
+    """
+    Read parquet file, sample specified number of rows from it and return a data frame.
+
+    :param file_path: path to the Parquet file.
+    :type file_path: str
+    :param sample_nrows: number of rows being sampled
+    :type sample_nrows: int
+    :param selected_columns: columns need to be read
+    :type selected_columns: list
+    :param read_in_string: return as string type
+    :type read_in_string: bool
+    :return:
+    :rtype: Iterator(pd.DataFrame)
+    """
+    # read parquet file into table
+    if selected_columns:
+        parquet_table = pq.read_table(file_path, columns=selected_columns)
+    else:
+        parquet_table = pq.read_table(file_path)
+
+    # sample
+    n_rows = parquet_table.num_rows
+    if n_rows > sample_nrows:
+        sample_index = np.array([False] * n_rows)
+        sample_index[random.sample(range(n_rows), sample_nrows)] = True
+    else:
+        sample_index = np.array([True] * n_rows)
+    sample_df = parquet_table.filter(sample_index).to_pandas()
+
+    # Convert all the unicode columns to utf-8
+    sample_df = convert_unicode_col_to_utf8(sample_df)
+
+    original_df_dtypes = sample_df.dtypes
+    if read_in_string:
+        sample_df = sample_df.astype(str)
+
+    return sample_df, original_df_dtypes
+
+
 def read_parquet_df(
     file_path: str,
+    sample_nrows: Optional[int] = None,
     selected_columns: Optional[List[str]] = None,
     read_in_string: bool = False,
 ) -> Tuple[pd.DataFrame, pd.Series]:
@@ -342,42 +529,42 @@ def read_parquet_df(
 
     :param file_path: path to the Parquet file.
     :type file_path: str
+    :param sample_nrows: number of rows being sampled
+    :type sample_nrows: int
+    :param selected_columns: columns need to be read
+    :type selected_columns: list
+    :param read_in_string: return as string type
+    :type read_in_string: bool
     :return:
     :rtype: Iterator(pd.DataFrame)
     """
-    parquet_file = pq.ParquetFile(file_path)
-    data = pd.DataFrame()
-    for i in range(parquet_file.num_row_groups):
+    if sample_nrows is None:
+        parquet_file = pq.ParquetFile(file_path)
+        data = pd.DataFrame()
+        for i in range(parquet_file.num_row_groups):
 
-        data_row_df = parquet_file.read_row_group(i).to_pandas()
+            data_row_df = parquet_file.read_row_group(i).to_pandas()
 
-        # Convert all the unicode columns to utf-8
-        types = data_row_df.apply(
-            lambda x: pd.api.types.infer_dtype(x.values, skipna=True)
+            # Convert all the unicode columns to utf-8
+            data_row_df = convert_unicode_col_to_utf8(data_row_df)
+
+            if selected_columns:
+                data_row_df = data_row_df[selected_columns]
+
+            data = pd.concat([data, data_row_df])
+
+        original_df_dtypes = data.dtypes
+        if read_in_string:
+            data = data.astype(str)
+        return data, original_df_dtypes
+    else:
+        data, original_df_dtypes = sample_parquet(
+            file_path,
+            sample_nrows,
+            selected_columns=selected_columns,
+            read_in_string=read_in_string,
         )
-
-        mixed_and_unicode_cols = types[types == "unicode"].index.union(
-            types[types == "mixed"].index
-        )
-
-        for col in mixed_and_unicode_cols:
-            data_row_df[col] = data_row_df[col].apply(
-                lambda x: x.encode("utf-8").strip() if isinstance(x, str) else x
-            )
-            data_row_df[col] = data_row_df[col].apply(
-                lambda x: x.decode("utf-8").strip() if isinstance(x, bytes) else x
-            )
-
-        if selected_columns:
-            data_row_df = data_row_df[selected_columns]
-
-        data = pd.concat([data, data_row_df])
-
-    original_df_dtypes = data.dtypes
-    if read_in_string:
-        data = data.astype(str)
-
-    return data, original_df_dtypes
+        return data, original_df_dtypes
 
 
 def read_text_as_list_of_strs(
@@ -446,7 +633,7 @@ def detect_file_encoding(
             # Try with small sample
             with FileOrBufferHandler(file_path, "rb") as input_file:
                 raw_data = input_file.read(10000)
-                result = from_bytes(
+                results = from_bytes(
                     raw_data,
                     steps=5,
                     chunk_size=512,
@@ -456,16 +643,15 @@ def detect_file_encoding(
                     preemptive_behaviour=True,
                     explain=False,
                 )
-                result = result.best()
-            if result:
-                if result.first():
-                    encoding = result.first().encoding
+                best_result = results.best()
+            if best_result:
+                encoding = best_result.encoding
 
             # Try again with full sample
             if not _decode_is_valid(encoding):
                 with FileOrBufferHandler(file_path, "rb") as input_file:
                     raw_data = input_file.read(max_lines * buffer_size)
-                    result = from_bytes(
+                    results = from_bytes(
                         raw_data,
                         steps=max_lines,
                         chunk_size=buffer_size,
@@ -475,10 +661,9 @@ def detect_file_encoding(
                         preemptive_behaviour=True,
                         explain=False,
                     )
-                    result = result.best()
-                if result:
-                    if result.first():
-                        encoding = result.first().encoding
+                    best_result = results.best()
+                if best_result:
+                    encoding = best_result.encoding
 
         except Exception:
             logger.info(
@@ -717,7 +902,6 @@ def url_to_bytes(url_as_string: Url, options: Dict) -> BytesIO:
                 "Content-length" in url.headers
                 and int(url.headers["Content-length"]) >= 1024**3
             ):
-
                 raise ValueError(
                     "The downloaded file from the url may not be " "larger than 1GB"
                 )
@@ -743,3 +927,125 @@ def url_to_bytes(url_as_string: Url, options: Dict) -> BytesIO:
 
     stream.seek(0)
     return stream
+
+
+class S3Helper:
+    """
+    A utility class for working with Amazon S3.
+
+    This class provides methods to check if a path is an S3 URI
+        and to create an S3 client.
+    """
+
+    @staticmethod
+    def is_s3_uri(path: str, logger: logging.Logger) -> bool:
+        """
+        Check if the given path is an S3 URI.
+
+        This function checks for common S3 URI prefixes "s3://" and "s3a://".
+
+        Args:
+            path (str): The path to check for an S3 URI.
+            logger (logging.Logger): The logger instance for logging.
+
+        Returns:
+            bool: True if the path is an S3 URI, False otherwise.
+        """
+        # Define the S3 URI prefixes to check
+        s3_uri_prefixes = ["s3://", "s3a://"]
+        path = path.strip()
+        # Check if the path starts with any of the specified prefixes
+        is_s3 = any(path.startswith(prefix) for prefix in s3_uri_prefixes)
+        if not is_s3:
+            logger.debug(f"'{path}' is not a valid S3 URI")
+
+        return is_s3
+
+    @staticmethod
+    def _create_boto3_client(
+        aws_access_key_id: Optional[str],
+        aws_secret_access_key: Optional[str],
+        aws_session_token: Optional[str],
+        region_name: Optional[str],
+    ) -> boto3.client:
+        return boto3.client(
+            "s3",
+            aws_access_key_id=aws_access_key_id,
+            aws_secret_access_key=aws_secret_access_key,
+            aws_session_token=aws_session_token,
+            region_name=region_name,
+        )
+
+    @staticmethod
+    def create_s3_client(
+        aws_access_key_id: Optional[str] = None,
+        aws_secret_access_key: Optional[str] = None,
+        aws_session_token: Optional[str] = None,
+        region_name: Optional[str] = None,
+    ) -> boto3.client:
+        """
+        Create and return an S3 client.
+
+        Args:
+            aws_access_key_id (str): The AWS access key ID.
+            aws_secret_access_key (str): The AWS secret access key.
+            aws_session_token (str): The AWS session token
+                (optional, typically used for temporary credentials).
+            region_name (str): The AWS region name (default is 'us-east-1').
+
+        Returns:
+            boto3.client: A S3 client instance.
+        """
+        # Check if credentials are not provided
+        # and use environment variables as fallback
+        if aws_access_key_id is None:
+            aws_access_key_id = os.environ.get("AWS_ACCESS_KEY_ID")
+        if aws_secret_access_key is None:
+            aws_secret_access_key = os.environ.get("AWS_SECRET_ACCESS_KEY")
+        if aws_session_token is None:
+            aws_session_token = os.environ.get("AWS_SESSION_TOKEN")
+
+        # Check if region is not provided and use environment variable as fallback
+        if region_name is None:
+            region_name = os.environ.get("AWS_REGION", "us-east-1")
+
+        # Check if IAM roles for service accounts are available
+        try:
+            s3 = S3Helper._create_boto3_client(
+                aws_access_key_id, aws_secret_access_key, aws_session_token, region_name
+            )
+        except botocore.exceptions.NoCredentialsError:
+            # IAM roles are not available, so fall back to provided credentials
+            if aws_access_key_id is None or aws_secret_access_key is None:
+                raise ValueError(
+                    "AWS access key ID and secret access key are required."
+                )
+            s3 = S3Helper._create_boto3_client(
+                aws_access_key_id, aws_secret_access_key, aws_session_token, region_name
+            )
+
+        return s3
+
+    @staticmethod
+    def get_s3_uri(s3_uri: str, s3_client: boto3.client) -> BytesIO:
+        """
+        Download an object from an S3 URI and return its content as BytesIO.
+
+        Args:
+            s3_uri (str): The S3 URI specifying the location of the object to download.
+            s3_client (boto3.client): An initialized AWS S3 client
+                for accessing the S3 service.
+
+        Returns:
+            BytesIO: A BytesIO object containing the content of
+                the downloaded S3 object.
+        """
+        # Parse the S3 URI
+        parsed_uri = urllib.parse.urlsplit(s3_uri)
+        bucket_name = parsed_uri.netloc
+        file_key = parsed_uri.path.lstrip("/")
+        # Download the S3 object
+        response = s3_client.get_object(Bucket=bucket_name, Key=file_key)
+
+        # Return the object's content as BytesIO
+        return BytesIO(response["Body"].read())
