@@ -4,23 +4,27 @@ from __future__ import annotations
 import abc
 from collections import OrderedDict
 from multiprocessing.pool import Pool
+from typing import Generic, TypeVar
 
 from pandas import Series
 
-from . import utils
+from . import profiler_utils
 from .categorical_column_profile import CategoricalColumn
 from .data_labeler_column_profile import DataLabelerColumn
 from .datetime_column_profile import DateTimeColumn
 from .float_column_profile import FloatColumn
 from .int_column_profile import IntColumn
+from .json_decoder import load_column_profile
 from .order_column_profile import OrderColumn
 from .profiler_options import BaseOption, StructuredOptions, UnstructuredOptions
 from .text_column_profile import TextColumn
 from .unstructured_labeler_profile import UnstructuredLabelerProfile
 from .unstructured_text_profile import TextProfiler
 
+BaseCompilerT = TypeVar("BaseCompilerT", bound="BaseCompiler")
 
-class BaseCompiler(metaclass=abc.ABCMeta):  # type: ignore
+
+class BaseCompiler(Generic[BaseCompilerT], metaclass=abc.ABCMeta):
     """Abstract class for generating a report."""
 
     # NOTE: these profilers are ordered. Test functionality if changed.
@@ -45,6 +49,7 @@ class BaseCompiler(metaclass=abc.ABCMeta):  # type: ignore
         if self._option_class is None:
             raise NotImplementedError("Must set the expected OptionClass.")
 
+        self.name = None
         self._profiles: dict = OrderedDict()
         if df_series is not None:
             self.name = df_series.name
@@ -101,7 +106,7 @@ class BaseCompiler(metaclass=abc.ABCMeta):  # type: ignore
                         df_series.name, options=profiler_options
                     )
                 except Exception as e:
-                    utils.warn_on_profile(profiler.type, e)
+                    profiler_utils.warn_on_profile(profiler.type, e)
 
         # Update profile after creation
         self.update_profile(df_series, pool)
@@ -139,7 +144,7 @@ class BaseCompiler(metaclass=abc.ABCMeta):  # type: ignore
             )
         return merged_profile_compiler
 
-    def diff(self, other: BaseCompiler, options: dict = None) -> dict:
+    def diff(self, other: BaseCompilerT, options: dict = None) -> dict:
         """
         Find the difference between 2 compilers and returns the report.
 
@@ -216,8 +221,40 @@ class BaseCompiler(metaclass=abc.ABCMeta):  # type: ignore
             self._profiles[profile_type].update(df_series)
         return self
 
+    @classmethod
+    def load_from_dict(cls, data, config: dict | None = None) -> BaseCompiler:
+        """
+        Parse attribute from json dictionary into self.
 
-class ColumnPrimitiveTypeProfileCompiler(BaseCompiler):
+        :param data: dictionary with attributes and values.
+        :type data: dict[string, Any]
+        :param config: config for loading column profiler params from dictionary
+        :type config: Dict | None
+
+        :return: Compiler with attributes populated.
+        :rtype: BaseCompiler
+        """
+        compiler = cls()
+
+        for attr, value in data.items():
+            if "_profiles" in attr:
+                for col_type, profile_as_dict in value.items():
+                    value[col_type] = load_column_profile(profile_as_dict, config)
+                # since needs to be in the same order, use _profilers to enforce
+                value = OrderedDict(
+                    {
+                        k.type: value[k.type]
+                        for k in compiler._profilers
+                        if k.type in value
+                    }
+                )
+            setattr(compiler, attr, value)
+        return compiler
+
+
+class ColumnPrimitiveTypeProfileCompiler(
+    BaseCompiler["ColumnPrimitiveTypeProfileCompiler"]
+):
     """For generating ordered column profile reports."""
 
     # NOTE: these profilers are ordered. Test functionality if changed.
@@ -301,7 +338,7 @@ class ColumnPrimitiveTypeProfileCompiler(BaseCompiler):
         if all_profiles:
             for key in all_profiles:
                 if key in self._profiles and key in other._profiles:
-                    diff = utils.find_diff_of_numbers(
+                    diff = profiler_utils.find_diff_of_numbers(
                         self._profiles[key].data_type_ratio,
                         other._profiles[key].data_type_ratio,
                     )
@@ -315,7 +352,7 @@ class ColumnPrimitiveTypeProfileCompiler(BaseCompiler):
         data_type1 = self.selected_data_type
         data_type2 = other.selected_data_type
         if data_type1 is not None or data_type2 is not None:
-            diff_profile["data_type"] = utils.find_diff_of_strings_and_bools(
+            diff_profile["data_type"] = profiler_utils.find_diff_of_strings_and_bools(
                 data_type1, data_type2
             )
             # Find diff of matching profile statistics
@@ -331,7 +368,7 @@ class ColumnPrimitiveTypeProfileCompiler(BaseCompiler):
         return diff_profile
 
 
-class ColumnStatsProfileCompiler(BaseCompiler):
+class ColumnStatsProfileCompiler(BaseCompiler["ColumnStatsProfileCompiler"]):
     """For generating OrderColumn and CategoricalColumn reports."""
 
     # NOTE: these profilers are ordered. Test functionality if changed.
@@ -376,7 +413,7 @@ class ColumnStatsProfileCompiler(BaseCompiler):
         return diff_profile
 
 
-class ColumnDataLabelerCompiler(BaseCompiler):
+class ColumnDataLabelerCompiler(BaseCompiler["ColumnDataLabelerCompiler"]):
     """For generating DataLabelerColumn report."""
 
     # NOTE: these profilers are ordered. Test functionality if changed.
@@ -427,7 +464,7 @@ class ColumnDataLabelerCompiler(BaseCompiler):
         return diff_profile
 
 
-class UnstructuredCompiler(BaseCompiler):
+class UnstructuredCompiler(BaseCompiler["UnstructuredCompiler"]):
     """For generating TextProfiler and UnstructuredLabelerProfile reports."""
 
     # NOTE: these profilers are ordered. Test functionality if changed.
