@@ -6,14 +6,14 @@ import abc
 import copy
 import itertools
 import warnings
-from typing import Any, Callable, Dict, List, cast
+from typing import Any, Callable, Dict, List, TypeVar, cast
 
 import numpy as np
 import numpy.typing as npt
 import pandas as pd
 import scipy.stats
 
-from . import histogram_utils, utils
+from . import float_column_profile, histogram_utils, profiler_utils
 from .base_column_profilers import BaseColumnProfiler
 from .profiler_options import NumericalOptions
 
@@ -31,7 +31,10 @@ class abstractstaticmethod(staticmethod):
     __isabstractmethod__ = True
 
 
-class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
+NumericStatsMixinT = TypeVar("NumericStatsMixinT", bound="NumericStatsMixin")
+
+
+class NumericStatsMixin(BaseColumnProfiler[NumericStatsMixinT], metaclass=abc.ABCMeta):
     """
     Abstract numerical column profile subclass of BaseColumnProfiler.
 
@@ -53,11 +56,11 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
                 "NumericalStatsMixin parameter 'options' must be "
                 "of type NumericalOptions."
             )
-        self.min: int | float | None = None
-        self.max: int | float | None = None
+        self.min: int | float | np.float64 | np.int64 | None = None
+        self.max: int | float | np.float64 | np.int64 | None = None
         self._top_k_modes: int = 5  # By default, return at max 5 modes
-        self.sum: int | float = 0
-        self._biased_variance: float = np.nan
+        self.sum: int | float | np.float64 | np.int64 = np.float64(0)
+        self._biased_variance: float | np.float64 = np.nan
         self._biased_skewness: float | np.float64 = np.nan
         self._biased_kurtosis: float | np.float64 = np.nan
         self._median_is_enabled: bool = True
@@ -77,14 +80,17 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         self.user_set_histogram_bin: int | None = None
         self.bias_correction: bool = True  # By default, we correct for bias
         self._mode_is_enabled: bool = True
-        self.num_zeros: int = 0
-        self.num_negatives: int = 0
+        self.num_zeros: int | np.int64 = np.int64(0)
+        self.num_negatives: int | np.int64 = np.int64(0)
+        self._num_quantiles: int = 1000  # By default, we use 1000 quantiles
+
         if options:
             self.bias_correction = options.bias_correction.is_enabled
             self._top_k_modes = options.mode.top_k_modes
             self._median_is_enabled = options.median.is_enabled
             self._median_abs_dev_is_enabled = options.median_abs_deviation.is_enabled
             self._mode_is_enabled = options.mode.is_enabled
+            self._num_quantiles = options.histogram_and_quantiles.num_quantiles
             bin_count_or_method = options.histogram_and_quantiles.bin_count_or_method
             if isinstance(bin_count_or_method, str):
                 self.histogram_bin_method_names = [bin_count_or_method]
@@ -95,23 +101,20 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
                 self.histogram_bin_method_names = ["custom"]
         self.histogram_methods: dict = {}
         self._stored_histogram: dict = {
-            "total_loss": 0,
-            "current_loss": 0,
+            "total_loss": np.float64(0.0),
+            "current_loss": np.float64(0.0),
             "suggested_bin_count": self.min_histogram_bin,
             "histogram": {"bin_counts": None, "bin_edges": None},
         }
         self._batch_history: list = []
         for method in self.histogram_bin_method_names:
             self.histogram_methods[method] = {
-                "total_loss": 0,
-                "current_loss": 0,
+                "total_loss": np.float64(0.0),
+                "current_loss": np.float64(0.0),
                 "suggested_bin_count": self.min_histogram_bin,
                 "histogram": {"bin_counts": None, "bin_edges": None},
             }
-        num_quantiles: int = 1000  # TODO: add to options
-        self.quantiles: list[float] | dict = {
-            bin_num: None for bin_num in range(num_quantiles - 1)
-        }
+        self.quantiles: list[float] | None = None
         self.__calculations = {
             "min": NumericStatsMixin._get_min,
             "max": NumericStatsMixin._get_max,
@@ -149,9 +152,9 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         Add histogram of two profiles together.
 
         :param other1: profile1 being added to self
-        :type other1: BaseColumnProfiler
+        :type other1: NumericStatsMixin
         :param other2: profile2 being added to self
-        :type other2: BaseColumnProfiler
+        :type other2: NumericStatsMixin
         :return: None
         """
         # get available bin methods and set to current
@@ -179,30 +182,91 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         # only the methods which intersect should exist.
         self.histogram_bin_method_names = bin_methods
         self.histogram_methods = dict()
+
+        # Set ideal bin count w/ if statement of some sort
+        ideal_count_of_bins = self.min_histogram_bin
+        if self.user_set_histogram_bin is not None:
+            ideal_count_of_bins = self.user_set_histogram_bin
+
         for method in self.histogram_bin_method_names:
             self.histogram_methods[method] = {
-                "total_loss": 0,
-                "current_loss": 0,
+                "total_loss": np.float64(0.0),
+                "current_loss": np.float64(0.0),
                 "histogram": {"bin_counts": None, "bin_edges": None},
             }
 
-        combined_values = np.concatenate(
-            [other1._histogram_to_array(), other2._histogram_to_array()]
+        # calculate the min of the first edge and the max of the last edge
+        # between two arrays
+        global_min_of_histogram_edges = (
+            float(self.min)
+            if self.min is not None
+            else min(
+                other1._stored_histogram["histogram"]["bin_edges"][0],
+                other2._stored_histogram["histogram"]["bin_edges"][0],
+            )
         )
-        bin_counts, bin_edges = self._get_histogram(combined_values)
-        self._stored_histogram["histogram"]["bin_counts"] = bin_counts
-        self._stored_histogram["histogram"]["bin_edges"] = bin_edges
 
-        histogram_loss = self._histogram_bin_error(combined_values)
-        self._stored_histogram["histogram"]["current_loss"] = histogram_loss
-        self._stored_histogram["histogram"]["total_loss"] = histogram_loss
+        global_max_of_histogram_edges = (
+            float(self.max)
+            if self.max is not None
+            else max(
+                other1._stored_histogram["histogram"]["bin_edges"][-1],
+                other2._stored_histogram["histogram"]["bin_edges"][-1],
+            )
+        )
+
+        # Generate new bin edges
+        ideal_bin_edges = np.linspace(
+            global_min_of_histogram_edges,
+            global_max_of_histogram_edges,
+            ideal_count_of_bins + 1,
+        )
+
+        # Initialize new count by bin object for population
+        new_entity_count_by_bin = np.zeros((ideal_count_of_bins,))
+
+        # Generate new histograms
+        _, hist_loss1 = self._assimilate_histogram(
+            from_hist_entity_count_per_bin=other1._stored_histogram["histogram"][
+                "bin_counts"
+            ],
+            from_hist_bin_edges=other1._stored_histogram["histogram"]["bin_edges"],
+            dest_hist_entity_count_per_bin=new_entity_count_by_bin,
+            dest_hist_bin_edges=ideal_bin_edges,
+            dest_hist_num_bin=ideal_count_of_bins,
+        )
+
+        # Ensure loss is calculated on second run of regenerate
+        _, hist_loss2 = self._assimilate_histogram(
+            from_hist_entity_count_per_bin=other2._stored_histogram["histogram"][
+                "bin_counts"
+            ],
+            from_hist_bin_edges=other2._stored_histogram["histogram"]["bin_edges"],
+            dest_hist_entity_count_per_bin=new_entity_count_by_bin,
+            dest_hist_bin_edges=ideal_bin_edges,
+            dest_hist_num_bin=ideal_count_of_bins,
+        )
+
+        aggregate_histogram_loss = hist_loss1 + hist_loss2
+
+        self._stored_histogram["histogram"]["bin_counts"] = new_entity_count_by_bin
+        self._stored_histogram["histogram"]["bin_edges"] = ideal_bin_edges
+
+        self._stored_histogram["histogram"]["current_loss"] = aggregate_histogram_loss
+        self._stored_histogram["histogram"]["total_loss"] = aggregate_histogram_loss
+
+        if self.user_set_histogram_bin is None:
+            for method in self.histogram_bin_method_names:
+                self.histogram_methods[method][
+                    "suggested_bin_count"
+                ] = histogram_utils._calculate_bins_from_profile(self, method)
 
         self._get_quantiles()
 
     def _add_helper(
         self,
-        other1: NumericStatsMixin,
-        other2: NumericStatsMixin,
+        other1: NumericStatsMixinT,
+        other2: NumericStatsMixinT,
     ) -> None:
         """
         Help merge profiles.
@@ -232,15 +296,6 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
                 other2._biased_variance,
                 other2.mean,
             )
-        if "histogram_and_quantiles" in self.__calculations.keys():
-            if other1._has_histogram and other2._has_histogram:
-                self._add_helper_merge_profile_histograms(other1, other2)
-            elif not other2._has_histogram:
-                self.histogram_methods = other1.histogram_methods
-                self.quantiles = other1.quantiles
-            else:
-                self.histogram_methods = other2.histogram_methods
-                self.quantiles = other2.quantiles
         if "min" in self.__calculations.keys():
             if other1.min is not None and other2.min is not None:
                 self.min = min(other1.min, other2.min)
@@ -287,6 +342,16 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         if "num_negatives" in self.__calculations.keys():
             self.num_negatives = other1.num_negatives + other2.num_negatives
 
+        if "histogram_and_quantiles" in self.__calculations.keys():
+            if other1._has_histogram and other2._has_histogram:
+                self._add_helper_merge_profile_histograms(other1, other2)
+            elif not other2._has_histogram:
+                self.histogram_methods = other1.histogram_methods
+                self.quantiles = other1.quantiles
+            else:
+                self.histogram_methods = other2.histogram_methods
+                self.quantiles = other2.quantiles
+
         # Merge max k mode count
         self._top_k_modes = max(other1._top_k_modes, other2._top_k_modes)
         # Merge median enable/disable option
@@ -360,9 +425,61 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
 
         return profile
 
+    def _reformat_numeric_stats_types_on_serialized_profiles(self):
+        """Assistance function in the deserialization of profiler objects.
+
+        This function is to be used to enforce correct typing for attributes
+        associated with the NumericStatsMixin conversions when loading profiler
+        objects in from their serialized saved format
+        """
+
+        def convert_histogram_key_types_to_np(histogram_info: dict):
+            if histogram_info["total_loss"] is not None:
+                histogram_info["total_loss"] = np.float64(histogram_info["total_loss"])
+
+            if histogram_info["current_loss"] is not None:
+                histogram_info["current_loss"] = np.float64(
+                    histogram_info["current_loss"]
+                )
+
+            # Convert hist lists to numpy arrays
+            for key in histogram_info["histogram"].keys():
+                if histogram_info["histogram"][key] is not None:
+                    histogram_info["histogram"][key] = np.array(
+                        histogram_info["histogram"][key]
+                    )
+            return histogram_info
+
+        self._stored_histogram = convert_histogram_key_types_to_np(
+            self._stored_histogram
+        )
+
+        # Convert hist method attributes to correct types
+        for key in self.histogram_methods.keys():
+            self.histogram_methods[key] = convert_histogram_key_types_to_np(
+                self.histogram_methods[key]
+            )
+
+        if self.min is not None:
+            self.min = np.float64(self.min)
+        if self.max is not None:
+            self.max = np.float64(self.max)
+        if self.sum is not None:
+            self.sum = np.float64(self.sum)
+        if self.num_zeros is not None:
+            self.num_zeros = np.int64(self.num_zeros)
+        if self.num_negatives is not None:
+            self.num_negatives = np.int64(self.num_negatives)
+        if not np.isnan(self._biased_variance):
+            self._biased_variance = np.float64(self._biased_variance)
+        if not np.isnan(self._biased_skewness):
+            self._biased_skewness = np.float64(self._biased_skewness)
+        if not np.isnan(self._biased_kurtosis):
+            self._biased_kurtosis = np.float64(self._biased_kurtosis)
+
     def diff(
         self,
-        other_profile: NumericStatsMixin,
+        other_profile: NumericStatsMixinT,
         options: dict = None,
     ) -> dict:
         """
@@ -381,20 +498,26 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
             )
 
         differences = {
-            "min": utils.find_diff_of_numbers(self.min, other_profile.min),
-            "max": utils.find_diff_of_numbers(self.max, other_profile.max),
-            "sum": utils.find_diff_of_numbers(self.sum, other_profile.sum),
-            "mean": utils.find_diff_of_numbers(self.mean, other_profile.mean),
-            "median": utils.find_diff_of_numbers(self.median, other_profile.median),
-            "mode": utils.find_diff_of_lists_and_sets(self.mode, other_profile.mode),
-            "median_absolute_deviation": utils.find_diff_of_numbers(
+            "min": profiler_utils.find_diff_of_numbers(self.min, other_profile.min),
+            "max": profiler_utils.find_diff_of_numbers(self.max, other_profile.max),
+            "sum": profiler_utils.find_diff_of_numbers(self.sum, other_profile.sum),
+            "mean": profiler_utils.find_diff_of_numbers(self.mean, other_profile.mean),
+            "median": profiler_utils.find_diff_of_numbers(
+                self.median, other_profile.median
+            ),
+            "mode": profiler_utils.find_diff_of_lists_and_sets(
+                self.mode, other_profile.mode
+            ),
+            "median_absolute_deviation": profiler_utils.find_diff_of_numbers(
                 self.median_abs_deviation,
                 other_profile.median_abs_deviation,
             ),
-            "variance": utils.find_diff_of_numbers(
+            "variance": profiler_utils.find_diff_of_numbers(
                 self.variance, other_profile.variance
             ),
-            "stddev": utils.find_diff_of_numbers(self.stddev, other_profile.stddev),
+            "stddev": profiler_utils.find_diff_of_numbers(
+                self.stddev, other_profile.stddev
+            ),
             "t-test": self._perform_t_test(
                 self.mean,
                 self.variance,
@@ -413,11 +536,11 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         return differences
 
     @property
-    def mean(self) -> float:
+    def mean(self) -> float | np.float64:
         """Return mean value."""
         if self.match_count == 0:
             return 0.0
-        return float(self.sum) / self.match_count
+        return self.sum / self.match_count
 
     @property
     def mode(self) -> list[float]:
@@ -444,7 +567,7 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         return self._get_percentile([50])[0]
 
     @property
-    def variance(self) -> float:
+    def variance(self) -> float | np.float64:
         """Return variance."""
         return (
             self._biased_variance
@@ -479,12 +602,17 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
 
     @staticmethod
     def _perform_t_test(
-        mean1: float, var1: float, n1: int, mean2: float, var2: float, n2: int
+        mean1: float | np.float64,
+        var1: float | np.float64,
+        n1: int,
+        mean2: float | np.float64,
+        var2: float | np.float64,
+        n2: int,
     ) -> dict:
         results: dict = {
             "t-statistic": None,
-            "conservative": {"df": None, "p-value": None},
-            "welch": {"df": None, "p-value": None},
+            "conservative": {"deg_of_free": None, "p-value": None},
+            "welch": {"deg_of_free": None, "p-value": None},
         }
 
         invalid_stats = False
@@ -494,7 +622,9 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
                 RuntimeWarning,
             )
             invalid_stats = True
-        if np.isnan([mean1, mean2, var1, var2]).any() or None in [
+        if np.isnan(
+            [float(mean1), float(mean2), float(var1), float(var2)]
+        ).any() or None in [
             mean1,
             mean2,
             var1,
@@ -517,17 +647,17 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
 
         s_delta = var1 / n1 + var2 / n2
         t = (mean1 - mean2) / np.sqrt(s_delta)
-        conservative_df = min(n1, n2) - 1
-        welch_df = s_delta**2 / (
+        conservative_deg_of_free = min(n1, n2) - 1
+        welch_deg_of_free = s_delta**2 / (
             (var1 / n1) ** 2 / (n1 - 1) + (var2 / n2) ** 2 / (n2 - 1)
         )
         results["t-statistic"] = t
-        results["conservative"]["df"] = float(conservative_df)
-        results["welch"]["df"] = float(welch_df)
+        results["conservative"]["deg_of_free"] = float(conservative_deg_of_free)
+        results["welch"]["deg_of_free"] = float(welch_deg_of_free)
 
-        conservative_t = scipy.stats.t(conservative_df)
+        conservative_t = scipy.stats.t(conservative_deg_of_free)
         conservative_p_val = (1 - conservative_t.cdf(abs(t))) * 2
-        welch_t = scipy.stats.t(welch_df)
+        welch_t = scipy.stats.t(welch_deg_of_free)
         welch_p_val = (1 - welch_t.cdf(abs(t))) * 2
 
         results["conservative"]["p-value"] = float(conservative_p_val)
@@ -577,9 +707,12 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
             # re-calculate `self` histogram
             if len_self_bin_counts != num_psi_bins:
                 histogram, hist_loss = self._regenerate_histogram(
-                    bin_counts=self_histogram["bin_counts"],
+                    entity_count_per_bin=self_histogram["bin_counts"],
                     bin_edges=self_histogram["bin_edges"],
                     suggested_bin_count=num_psi_bins,
+                    is_float_histogram=isinstance(
+                        self, float_column_profile.FloatColumn
+                    ),
                     options={
                         "min_edge": min_min_edge,
                         "max_edge": max_max_edge,
@@ -598,9 +731,12 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
 
             if histogram_edges_not_equal:
                 histogram, hist_loss = self._regenerate_histogram(
-                    bin_counts=other_histogram["bin_counts"],
+                    entity_count_per_bin=other_histogram["bin_counts"],
                     bin_edges=other_histogram["bin_edges"],
                     suggested_bin_count=num_psi_bins,
+                    is_float_histogram=isinstance(
+                        self, float_column_profile.FloatColumn
+                    ),
                     options={
                         "min_edge": min_min_edge,
                         "max_edge": max_max_edge,
@@ -684,7 +820,7 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         batch_mean: float,
         batch_var: float,
         batch_count: int,
-    ) -> float:
+    ) -> float | np.float64:
         """
         Calculate combined biased variance of the current values and new dataset.
 
@@ -692,7 +828,7 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         :param batch_var: biased variance of new chunk
         :param batch_count: number of samples in new chunk
         :return: combined biased variance
-        :rtype: float
+        :rtype: float | np.float64
         """
         return self._merge_biased_variance(
             self.match_count,
@@ -706,12 +842,12 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
     @staticmethod
     def _merge_biased_variance(
         match_count1: int,
-        biased_variance1: float,
-        mean1: float,
+        biased_variance1: float | np.float64,
+        mean1: float | np.float64,
         match_count2: int,
-        biased_variance2: float,
-        mean2: float,
-    ) -> float:
+        biased_variance2: float | np.float64,
+        mean2: float | np.float64,
+    ) -> float | np.float64:
         """
         Calculate combined biased variance of the current values and new dataset.
 
@@ -722,7 +858,7 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         :param mean2: mean of chunk 2
         :param biased_variance2: variance of chunk 2 without bias correction
         :return: combined variance
-        :rtype: float
+        :rtype: float | np.float64
         """
         if match_count1 < 1:
             return biased_variance2
@@ -744,7 +880,9 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         return new_variance
 
     @staticmethod
-    def _correct_bias_variance(match_count: int, biased_variance: float) -> float:
+    def _correct_bias_variance(
+        match_count: int, biased_variance: float | np.float64
+    ) -> float | np.float64:
         if match_count is None or biased_variance is None or match_count < 2:
             warnings.warn(
                 "Insufficient match count to correct bias in variance. Bias correction "
@@ -761,12 +899,12 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
     def _merge_biased_skewness(
         match_count1: int,
         biased_skewness1: float | np.float64,
-        biased_variance1: float,
-        mean1: float,
+        biased_variance1: float | np.float64,
+        mean1: float | np.float64,
         match_count2: int,
         biased_skewness2: float | np.float64,
-        biased_variance2: float,
-        mean2: float,
+        biased_variance2: float | np.float64,
+        mean2: float | np.float64,
     ) -> float | np.float64:
         """
         Calculate the combined skewness of two data chunks.
@@ -847,13 +985,13 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         match_count1: int,
         biased_kurtosis1: float | np.float64,
         biased_skewness1: float | np.float64,
-        biased_variance1: float,
-        mean1: float,
+        biased_variance1: float | np.float64,
+        mean1: float | np.float64,
         match_count2: int,
         biased_kurtosis2: float | np.float64,
         biased_skewness2: float | np.float64,
-        biased_variance2: float,
-        mean2: float,
+        biased_variance2: float | np.float64,
+        mean2: float | np.float64,
     ) -> float | np.float64:
         """
         Calculate the combined kurtosis of two sets of data.
@@ -1004,7 +1142,7 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
             sum_var += bin_var
         return sum_var
 
-    def _histogram_bin_error(self, input_array: np.ndarray | pd.Series) -> float:
+    def _histogram_bin_error(self, input_array: np.ndarray | pd.Series) -> np.float64:
         """
         Calculate error of each value from bin of the histogram it falls within.
 
@@ -1032,7 +1170,7 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
             (input_array - (bin_edges[inds] + bin_edges[inds - 1]) / 2) ** 2
         )
 
-        return sum_error
+        return np.float64(sum_error)
 
     @staticmethod
     def _histogram_loss(
@@ -1042,7 +1180,7 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         avg_totalvar: float,
         run_time: float,
         avg_runtime: float,
-    ) -> float:
+    ) -> np.float64:
 
         norm_diff_var: float = 0
         norm_total_var: float = 0
@@ -1054,7 +1192,7 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         penalized_time = 1  # currently set as 1s
         if (run_time - avg_runtime) >= penalized_time:
             norm_runtime = float(run_time - avg_runtime) / avg_runtime
-        return norm_diff_var + norm_total_var + norm_runtime
+        return np.float64(norm_diff_var + norm_total_var + norm_runtime)
 
     def _select_method_for_histogram(
         self,
@@ -1228,7 +1366,12 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         self._stored_histogram["total_loss"] += histogram_loss
 
     def _regenerate_histogram(
-        self, bin_counts, bin_edges, suggested_bin_count, options=None
+        self,
+        entity_count_per_bin,
+        bin_edges,
+        suggested_bin_count,
+        is_float_histogram,
+        options=None,
     ) -> tuple[dict[str, np.ndarray], float]:
 
         # create proper binning
@@ -1241,34 +1384,69 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
                 options["min_edge"], options["max_edge"], suggested_bin_count + 1
             )
 
+        # if it's not a float histogram, then assume it only contains integer values
+        if not is_float_histogram:
+            bin_edges = np.round(bin_edges)
+
+        return self._assimilate_histogram(
+            from_hist_entity_count_per_bin=entity_count_per_bin,
+            from_hist_bin_edges=bin_edges,
+            dest_hist_entity_count_per_bin=new_bin_counts,
+            dest_hist_bin_edges=new_bin_edges,
+            dest_hist_num_bin=suggested_bin_count,
+        )
+
+    def _assimilate_histogram(
+        self,
+        from_hist_entity_count_per_bin: np.ndarray,
+        from_hist_bin_edges: np.ndarray,
+        dest_hist_entity_count_per_bin: np.ndarray,
+        dest_hist_bin_edges: np.ndarray,
+        dest_hist_num_bin: int,
+    ) -> tuple[dict[str, np.ndarray[Any, Any]], float]:
+        """
+        Assimilates a histogram into another histogram using specifications.
+
+        :param from_hist_entity_count_per_bin: the breakdown of number of entities
+            within a given histogram bin (to be assimilated)
+        :type from_hist_entity_count_per_bin: List[float]
+        :param from_hist_bin_edges: List of value ranges for histogram bins
+            (to be assimilated)
+        :type from_hist_bin_edges: List[Tuple[float]]
+        :param from_hist_entity_count_per_bin: the breakdown of number of
+            entities within a given histogram bin (assimilated to)
+        :type dest_hist_entity_count_per_bin: List[float]
+        :param dest_hist_bin_edges: List of value ranges for histogram bins
+            (assimilated to)
+        :type dest_hist_bin_edges: List[Tuple[float]]
+        :type dest_hist_bin_edges: List[Tuple[float]
+        :param dest_hist_num_bin: The number of bins desired for histogram
+        :type dest_hist_num_bin: int
+        :return: Tuple containing dictionary of histogram info and histogram loss
+        """
         # allocate bin_counts
         new_bin_id = 0
         hist_loss = 0
-        for bin_id, bin_count in enumerate(bin_counts):
+        for bin_id, bin_count in enumerate(from_hist_entity_count_per_bin):
             if not bin_count:  # if nothing in bin, nothing to add
                 continue
 
-            bin_edge = bin_edges[bin_id : bin_id + 3]
-
-            # if we know not float, we can assume values in bins are integers.
-            is_float_profile = self.__class__.__name__ == "FloatColumn"
-            if not is_float_profile:
-                bin_edge = np.round(bin_edge)
+            bin_edge = from_hist_bin_edges[bin_id : bin_id + 3]
 
             # loop until we have a new bin which contains the current bin.
             while (
-                bin_edge[0] >= new_bin_edges[new_bin_id + 1]
-                and new_bin_id < suggested_bin_count - 1
+                bin_edge[0] >= dest_hist_bin_edges[new_bin_id + 1]
+                and new_bin_id < dest_hist_num_bin - 1
             ):
                 new_bin_id += 1
 
-            new_bin_edge = new_bin_edges[new_bin_id : new_bin_id + 3]
+            new_bin_edge = dest_hist_bin_edges[new_bin_id : new_bin_id + 3]
 
             # find where the current bin falls within the new bins
-            is_last_bin = new_bin_id == suggested_bin_count - 1
+            is_last_bin = new_bin_id == dest_hist_num_bin - 1
             if bin_edge[1] < new_bin_edge[1] or is_last_bin:
                 # current bin is within the new bin
-                new_bin_counts[new_bin_id] += bin_count
+                dest_hist_entity_count_per_bin[new_bin_id] += bin_count
                 hist_loss += (
                     ((new_bin_edge[1] + new_bin_edge[0]) - (bin_edge[1] + bin_edge[0]))
                     / 2
@@ -1280,23 +1458,30 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
                     bin_edge[1] - bin_edge[0]
                 )
                 count_in_left_bin = round(bin_count * percentage_in_left_bin)
-                new_bin_counts[new_bin_id] += count_in_left_bin
+                dest_hist_entity_count_per_bin[new_bin_id] += count_in_left_bin
                 hist_loss += (
                     ((new_bin_edge[1] + new_bin_edge[0]) - (bin_edge[1] + bin_edge[0]))
                     / 2
                 ) ** 2 * count_in_left_bin
 
                 # allocate leftovers to the right bin
-                new_bin_counts[new_bin_id + 1] += bin_count - count_in_left_bin
+                dest_hist_entity_count_per_bin[new_bin_id + 1] += (
+                    bin_count - count_in_left_bin
+                )
                 hist_loss += (
-                    ((new_bin_edge[2] - new_bin_edge[1]) - (bin_edge[1] - bin_edge[0]))
+                    ((new_bin_edge[2] + new_bin_edge[1]) - (bin_edge[1] + bin_edge[0]))
                     / 2
                 ) ** 2 * (bin_count - count_in_left_bin)
 
                 # increment bin id to the right bin
                 new_bin_id += 1
-
-        return ({"bin_edges": new_bin_edges, "bin_counts": new_bin_counts}, hist_loss)
+        return (
+            {
+                "bin_edges": dest_hist_bin_edges,
+                "bin_counts": dest_hist_entity_count_per_bin,
+            },
+            hist_loss,
+        )
 
     def _histogram_for_profile(
         self, histogram_method: str
@@ -1336,9 +1521,10 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
             )
 
         return self._regenerate_histogram(
-            bin_counts=bin_counts,
+            entity_count_per_bin=bin_counts,
             bin_edges=bin_edges,
             suggested_bin_count=suggested_bin_count,
+            is_float_histogram=isinstance(self, float_column_profile.FloatColumn),
         )
 
     def _get_best_histogram_for_profile(self) -> dict:
@@ -1540,7 +1726,9 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
 
         :return: list of quantiles
         """
-        percentiles: np.ndarray = np.linspace(0, 100, len(self.quantiles) + 2)[1:-1]
+        percentiles: np.ndarray = np.linspace(0, 100, (self._num_quantiles - 1) + 2)[
+            1:-1
+        ]
         self.quantiles = self._get_percentile(percentiles=percentiles)
 
     def _update_helper(self, df_series_clean: pd.Series, profile: dict) -> None:
@@ -1675,7 +1863,7 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         ):
             return
 
-        batch_biased_skewness = utils.biased_skew(df_series)
+        batch_biased_skewness = profiler_utils.biased_skew(df_series)
         subset_properties["biased_skewness"] = batch_biased_skewness
         batch_count = subset_properties["match_count"]
         batch_biased_var = subset_properties["biased_variance"]
@@ -1719,7 +1907,7 @@ class NumericStatsMixin(metaclass=abc.ABCMeta):  # type: ignore
         ):
             return
 
-        batch_biased_kurtosis = utils.biased_kurt(df_series)
+        batch_biased_kurtosis = profiler_utils.biased_kurt(df_series)
         subset_properties["biased_kurtosis"] = batch_biased_kurtosis
         batch_count = subset_properties["match_count"]
         batch_biased_var = subset_properties["biased_variance"]
