@@ -1,11 +1,13 @@
 import os
 import unittest
 from unittest import mock
+from unittest.mock import MagicMock, patch
 
 import pandas as pd
 import requests
 
 from dataprofiler.data_readers.data import Data
+from dataprofiler.data_readers.text_data import TextData
 
 test_root_path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 
@@ -160,6 +162,35 @@ class TestDataReadFromURL(unittest.TestCase):
         ):
             data_obj = Data("https://test.com")
 
+    @patch("boto3.client")
+    @patch(
+        "os.environ",
+        {
+            "AWS_ACCESS_KEY_ID": "<YOUR_ACCESS_KEY>",
+            "AWS_SECRET_ACCESS_KEY": "<YOUR_SECRET_KEY>",
+        },
+    )
+    def test_read_s3_uri(self, mock_boto3_client):
+        region_name = "us-east-1"
+
+        # Create a custom mock response
+        custom_response = {"Body": MagicMock()}
+        custom_response["Body"].read.return_value = b"Test S3 content"
+
+        # Mock the behavior of the S3 client to return the custom response
+        mock_boto3_client.return_value.get_object.return_value = custom_response
+        data_obj = Data("s3a://my-bucket/my_file.txt")
+
+        self.assertEqual(type(data_obj), TextData)
+
+        mock_boto3_client.assert_called_with(
+            "s3",
+            aws_access_key_id="<YOUR_ACCESS_KEY>",
+            aws_secret_access_key="<YOUR_SECRET_KEY>",
+            aws_session_token=None,
+            region_name=region_name,
+        )
+
 
 if __name__ == "__main__":
     unittest.main()
