@@ -1,3 +1,6 @@
+import json
+
+from dataprofiler.profilers.json_encoder import ProfileEncoder
 from dataprofiler.profilers.profiler_options import BaseInspectorOptions
 from dataprofiler.tests.profilers.profiler_options.test_boolean_option import (
     TestBooleanOption,
@@ -47,3 +50,15 @@ class TestBaseInspectorOptions(TestBooleanOption):
 
     def test_eq(self):
         super().test_eq()
+
+    def test_json_encode(self):
+        option = BaseInspectorOptions(is_enabled=False)
+
+        serialized = json.dumps(option, cls=ProfileEncoder)
+
+        expected = {
+            "class": "BaseInspectorOptions",
+            "data": {"is_enabled": False},
+        }
+
+        self.assertDictEqual(expected, json.loads(serialized))
