@@ -1,7 +1,12 @@
+pass
+import json
 from unittest import mock
 
 from dataprofiler.labelers.base_data_labeler import BaseDataLabeler
+from dataprofiler.profilers.json_decoder import load_option
+from dataprofiler.profilers.json_encoder import ProfileEncoder
 from dataprofiler.profilers.profiler_options import DataLabelerOptions
+from dataprofiler.tests.profilers import utils as test_utils
 from dataprofiler.tests.profilers.profiler_options.test_base_inspector_options import (
     TestBaseInspectorOptions,
 )
@@ -168,3 +173,77 @@ class TestDataLabelerOptions(TestBaseInspectorOptions):
         self.assertNotEqual(options, options2)
         options2.data_labeler_object._model = 7
         self.assertEqual(options, options2)
+
+    def test_json_encode(self):
+        option = DataLabelerOptions()
+
+        mock_BaseDataLabeler = mock.Mock(spec=BaseDataLabeler)
+        mock_BaseDataLabeler._default_model_loc = "test_loc"
+        option.data_labeler_object = mock_BaseDataLabeler
+
+        serialized = json.dumps(option, cls=ProfileEncoder)
+
+        expected = {
+            "class": "DataLabelerOptions",
+            "data": {
+                "is_enabled": True,
+                "data_labeler_dirpath": None,
+                "max_sample_size": None,
+                "data_labeler_object": {"from_library": "test_loc"},
+            },
+        }
+
+        self.assertDictEqual(expected, json.loads(serialized))
+
+    @mock.patch(
+        "dataprofiler.profilers.profiler_utils.DataLabeler",
+        spec=BaseDataLabeler,
+    )
+    def test_json_decode(self, mock_BaseDataLabeler):
+        expected_options = self.get_options()
+
+        serialized = json.dumps(expected_options, cls=ProfileEncoder)
+        deserialized = load_option(json.loads(serialized))
+
+        test_utils.assert_profiles_equal(deserialized, expected_options)
+
+        # case where labeler exists but no config
+        mock_BaseDataLabeler._default_model_loc = "test_loc"
+        expected_options.data_labeler_object = mock_BaseDataLabeler
+        mock_BaseDataLabeler.load_from_library.return_value = mock_BaseDataLabeler
+        config = {}
+
+        serialized = json.dumps(expected_options, cls=ProfileEncoder)
+        deserialized = load_option(json.loads(serialized), config)
+        test_utils.assert_profiles_equal(deserialized, expected_options)
+
+        expected_config = {
+            "DataLabelerOptions": {"from_library": {"test_loc": mock_BaseDataLabeler}},
+            "DataLabelerColumn": {"from_library": {"test_loc": mock_BaseDataLabeler}},
+        }
+        self.assertDictEqual(expected_config, config)
+
+        mock_BaseDataLabeler.load_from_library.reset_mock()
+        mock_BaseDataLabeler.load_from_library.return_value = None
+        deserialized = load_option(json.loads(serialized), config)
+
+        mock_BaseDataLabeler.load_from_library.assert_not_called()
+        test_utils.assert_profiles_equal(deserialized, expected_options)
+
+        config = {
+            "DataLabelerColumn": {"from_library": {"test_loc": mock_BaseDataLabeler}}
+        }
+        deserialized = load_option(json.loads(serialized), config)
+
+        mock_BaseDataLabeler.load_from_library.assert_not_called()
+        test_utils.assert_profiles_equal(deserialized, expected_options)
+        self.assertDictEqual(expected_config, config)
+
+        config = {
+            "DataLabelerOptions": {"from_library": {"test_loc": mock_BaseDataLabeler}}
+        }
+        deserialized = load_option(json.loads(serialized), config)
+
+        mock_BaseDataLabeler.load_from_library.assert_not_called()
+        test_utils.assert_profiles_equal(deserialized, expected_options)
+        self.assertDictEqual(expected_config, config)
