@@ -1,12 +1,18 @@
+import json
 import re
+from unittest import mock
 
+from dataprofiler.profilers.json_encoder import ProfileEncoder
 from dataprofiler.profilers.profiler_options import StructuredOptions
+from dataprofiler.tests.profilers.profiler_options.abstract_test_options import (
+    JSONDecodeTestMixin,
+)
 from dataprofiler.tests.profilers.profiler_options.test_base_option import (
     TestBaseOption,
 )
 
 
-class TestStructuredOptions(TestBaseOption):
+class TestStructuredOptions(TestBaseOption, JSONDecodeTestMixin):
 
     option_class = StructuredOptions
     other_keys = ["null_values", "column_null_values"]
@@ -21,6 +27,7 @@ class TestStructuredOptions(TestBaseOption):
         "multiprocess",
         "correlation",
         "chi2_homogeneity",
+        "row_statistics",
     ]
     keys = boolean_keys + other_keys
 
@@ -91,6 +98,10 @@ class TestStructuredOptions(TestBaseOption):
             option.set({"column_null_values": test_dict})
             self.assertEqual(test_dict, option.column_null_values)
 
+        for test_val in [0.2, 1]:
+            option.set({"sampling_ratio": test_val})
+            self.assertEqual(test_val, option.sampling_ratio)
+
     def test_validate_helper(self):
         # Valid cases should return [] while invalid cases
         # should return a list of errors
@@ -137,6 +148,7 @@ class TestStructuredOptions(TestBaseOption):
         option.multiprocess = StructuredOptions()
         option.correlation = StructuredOptions()
         option.chi2_homogeneity = StructuredOptions()
+        option.row_statistics = StructuredOptions()
 
         expected_error = set()
         for key in self.boolean_keys:
@@ -147,6 +159,8 @@ class TestStructuredOptions(TestBaseOption):
                 ckey = "Categorical"
             elif key == "datetime":
                 ckey = "DateTime"
+            elif key == "row_statistics":
+                ckey = "RowStatistics"
             if key == "multiprocess" or key == "chi2_homogeneity":
                 expected_error.add(f"{optpth}.{key} must be a(n) BooleanOption.")
             else:
@@ -201,6 +215,7 @@ class TestStructuredOptions(TestBaseOption):
         option.multiprocess = StructuredOptions()
         option.correlation = StructuredOptions()
         option.chi2_homogeneity = StructuredOptions()
+        option.row_statistics = StructuredOptions()
 
         expected_error = set()
         for key in self.boolean_keys:
@@ -211,6 +226,8 @@ class TestStructuredOptions(TestBaseOption):
                 ckey = "Categorical"
             elif key == "datetime":
                 ckey = "DateTime"
+            elif key == "row_statistics":
+                ckey = "RowStatistics"
             if key == "multiprocess" or key == "chi2_homogeneity":
                 expected_error.add(f"{optpth}.{key} must be a(n) BooleanOption.")
             else:
@@ -276,6 +293,32 @@ class TestStructuredOptions(TestBaseOption):
         option.set({"column_null_values": None})
         self.assertEqual([], option._validate_helper())
 
+        expected_error_type = [f"{optpth}.sampling_ratio must be a float or an integer"]
+        expected_error_value = [
+            "{}.sampling_ratio must be greater than 0.0 and less than or equal to 1.0".format(
+                optpth
+            )
+        ]
+        expected_error_none_value = [f"{optpth}.sampling_ratio may not be None"]
+        # Test ratio is None
+        option.set({"sampling_ratio": None})
+        self.assertEqual(expected_error_none_value, option._validate_helper())
+        # Test ratio is not a float
+        option.set({"sampling_ratio": "1"})
+        self.assertEqual(expected_error_type, option._validate_helper())
+        # Test ratio is greater than upper bound
+        option.set({"sampling_ratio": 2.5})
+        self.assertEqual(expected_error_value, option._validate_helper())
+        # Test ratio is greater than upper bound, int
+        option.set({"sampling_ratio": 3})
+        self.assertEqual(expected_error_value, option._validate_helper())
+        # Test ratio is lesser than lower bound
+        option.set({"sampling_ratio": -2.5})
+        self.assertEqual(expected_error_value, option._validate_helper())
+        # Test ratio is lesser than lower bound, int
+        option.set({"sampling_ratio": -5})
+        self.assertEqual(expected_error_value, option._validate_helper())
+
     def test_enabled_profilers(self):
         options = self.get_options()
         self.assertNotIn("null_values", options.enabled_profiles)
@@ -313,3 +356,66 @@ class TestStructuredOptions(TestBaseOption):
         self.assertNotEqual(options, options2)
         options2.float.precision.sample_ratio = 0.1
         self.assertEqual(options, options2)
+
+    def test_json_encode(self):
+        option = StructuredOptions(
+            null_values={"str": 1}, column_null_values={2: {"other_str": 5}}
+        )
+
+        serialized = json.dumps(option, cls=ProfileEncoder)
+
+        expected = {
+            "class": "StructuredOptions",
+            "data": {
+                "sampling_ratio": 0.2,
+                "multiprocess": {
+                    "class": "BooleanOption",
+                    "data": {"is_enabled": True},
+                },
+                "int": {
+                    "class": "IntOptions",
+                    "data": mock.ANY,
+                },
+                "float": {
+                    "class": "FloatOptions",
+                    "data": mock.ANY,
+                },
+                "datetime": {
+                    "class": "DateTimeOptions",
+                    "data": {"is_enabled": True},
+                },
+                "text": {
+                    "class": "TextOptions",
+                    "data": mock.ANY,
+                },
+                "order": {"class": "OrderOptions", "data": {"is_enabled": True}},
+                "category": {
+                    "class": "CategoricalOptions",
+                    "data": mock.ANY,
+                },
+                "data_labeler": {
+                    "class": "DataLabelerOptions",
+                    "data": mock.ANY,
+                },
+                "correlation": {
+                    "class": "CorrelationOptions",
+                    "data": mock.ANY,
+                },
+                "chi2_homogeneity": {
+                    "class": "BooleanOption",
+                    "data": {"is_enabled": True},
+                },
+                "null_replication_metrics": {
+                    "class": "BooleanOption",
+                    "data": {"is_enabled": False},
+                },
+                "row_statistics": {
+                    "class": "RowStatisticsOptions",
+                    "data": mock.ANY,
+                },
+                "null_values": {"str": 1},
+                "column_null_values": {"2": {"other_str": 5}},
+            },
+        }
+
+        self.assertDictEqual(expected, json.loads(serialized))
