@@ -1,3 +1,4 @@
+import json
 import unittest
 from collections import defaultdict
 from unittest import mock
@@ -5,12 +6,20 @@ from unittest import mock
 import numpy as np
 import pandas as pd
 
-from dataprofiler.profilers import utils
+from dataprofiler.labelers import BaseDataLabeler
+from dataprofiler.profilers import profiler_utils
 from dataprofiler.profilers.data_labeler_column_profile import DataLabelerColumn
+from dataprofiler.profilers.json_decoder import load_column_profile
+from dataprofiler.profilers.json_encoder import ProfileEncoder
 from dataprofiler.profilers.profiler_options import DataLabelerOptions
 
+from . import utils as test_utils
 
-@mock.patch("dataprofiler.profilers.data_labeler_column_profile.DataLabeler")
+
+@mock.patch(
+    "dataprofiler.profilers.data_labeler_column_profile.DataLabeler",
+    spec=BaseDataLabeler,
+)
 class TestDataLabelerColumnProfiler(unittest.TestCase):
     @staticmethod
     def _setup_data_labeler_mock(mock_instance):
@@ -19,6 +28,7 @@ class TestDataLabelerColumnProfiler(unittest.TestCase):
         mock_DataLabeler.reverse_label_mapping = {0: "a", 1: "b"}
         mock_DataLabeler.model.num_labels = 2
         mock_DataLabeler.model.requires_zero_mapping = False
+        mock_instance.load_from_library.side_effect = mock_instance
 
         def mock_predict(data, *args, **kwargs):
             len_data = len(data)
@@ -366,13 +376,12 @@ class TestDataLabelerColumnProfiler(unittest.TestCase):
 
             diff = profiler1.diff(profiler2)
             expected_diff = {
-                "data_label": utils.find_diff_of_lists_and_sets(
+                "data_label": profiler_utils.find_diff_of_lists_and_sets(
                     ["a", "b", "c"], ["b", "c", "d"]
                 ),
                 "avg_predictions": {"a": "unchanged", "b": -0.70, "c": 0.70},
                 "label_representation": {"a": -0.84, "b": "unchanged", "c": 0.84},
             }
-            self.maxDiff = None
             self.assertDictEqual(expected_diff, diff)
 
     def test_empty_data(self, *mocks):
@@ -391,3 +400,158 @@ class TestDataLabelerColumnProfiler(unittest.TestCase):
 
         diff_profile = profiler1.diff(profiler2)
         self.assertIsNone(merge_profile.data_label)
+
+    def test_json_encode(self, mock_instance):
+        self._setup_data_labeler_mock(mock_instance)
+        profiler = DataLabelerColumn("")
+
+        # Validates that error is raised if model loc is not set for labeler
+        profiler.data_labeler._default_model_loc = None
+        with self.assertRaisesRegex(
+            ValueError,
+            "Serialization cannot be done on labelers with _default_model_loc not set",
+        ):
+            _ = json.dumps(profiler, cls=ProfileEncoder)
+
+        # Reset the model loc to its initial value
+        profiler.data_labeler._default_model_loc = "this is a test model loc"
+        serialized = json.dumps(profiler, cls=ProfileEncoder)
+
+        expected = json.dumps(
+            {
+                "class": "DataLabelerColumn",
+                "data": {
+                    "name": "",
+                    "col_index": float("nan"),
+                    "sample_size": 0,
+                    "metadata": {},
+                    "times": {},
+                    "thread_safe": False,
+                    "_max_sample_size": 1000,
+                    "data_labeler": {"from_library": "this is a test model loc"},
+                    "_reverse_label_mapping": None,
+                    "_possible_data_labels": None,
+                    "_rank_distribution": None,
+                    "_sum_predictions": None,
+                    "_top_k_voting": 1,
+                    "_min_voting_prob": 0.2,
+                    "_min_prob_differential": 0.2,
+                    "_top_k_labels": 3,
+                    "_min_top_label_prob": 0.35,
+                    "_DataLabelerColumn__calculations": {},
+                },
+            }
+        )
+
+        self.assertEqual(serialized, expected)
+
+    def test_json_encode_after_update(self, mock_instance):
+        self._setup_data_labeler_mock(mock_instance)
+        data = pd.Series(["1", "2", "3", "4"], dtype=object)
+        profiler = DataLabelerColumn(data.name)
+        profiler.data_labeler._default_model_loc = "this is a test model loc"
+        with test_utils.mock_timeit():
+            profiler.update(data)
+
+        serialized = json.dumps(profiler, cls=ProfileEncoder)
+        expected = json.dumps(
+            {
+                "class": "DataLabelerColumn",
+                "data": {
+                    "name": None,
+                    "col_index": float("nan"),
+                    "sample_size": 4,
+                    "metadata": {},
+                    "times": {"data_labeler_predict": 1.0},
+                    "thread_safe": False,
+                    "_max_sample_size": 1000,
+                    "data_labeler": {"from_library": "this is a test model loc"},
+                    "_reverse_label_mapping": {0: "a", 1: "b"},
+                    "_possible_data_labels": ["a", "b"],
+                    "_rank_distribution": {
+                        "a": 2,
+                        "b": 2,
+                    },
+                    "_sum_predictions": [2.0, 2.0],
+                    "_top_k_voting": 1,
+                    "_min_voting_prob": 0.2,
+                    "_min_prob_differential": 0.2,
+                    "_top_k_labels": 3,
+                    "_min_top_label_prob": 0.35,
+                    "_DataLabelerColumn__calculations": {},
+                },
+            }
+        )
+
+        self.assertEqual(expected, serialized)
+
+    @mock.patch(
+        "dataprofiler.profilers.profiler_utils.DataLabeler", spec=BaseDataLabeler
+    )
+    def test_json_decode(self, mock_utils_DataLabeler, mock_BaseDataLabeler):
+        self._setup_data_labeler_mock(mock_BaseDataLabeler)
+        mock_utils_DataLabeler.load_from_library.side_effect = mock_BaseDataLabeler
+
+        data = pd.Series(["1", "2", "3", "4"], dtype=object)
+        expected = DataLabelerColumn(data.name)
+        expected.data_labeler._default_model_loc = "structured_model"
+        serialized = json.dumps(expected, cls=ProfileEncoder)
+
+        deserialized = load_column_profile(json.loads(serialized))
+
+        test_utils.assert_profiles_equal(deserialized, expected)
+
+        # test decode with options to override load labeler
+        # create a new labeler ot load instead of from_library
+        new_mock_data_labeler = mock.Mock(spec=BaseDataLabeler)
+        new_mock_data_labeler.name = "new fake data labeler"
+        new_mock_data_labeler._default_model_loc = "my/fake/path"
+        config = {
+            "DataLabelerColumn": {
+                "from_library": {"structured_model": new_mock_data_labeler}
+            }
+        }
+
+        mock_BaseDataLabeler.reset_mock()  # set to 0 calls as option should override
+        mock_utils_DataLabeler.reset_mock()  # set to 0 calls as option should override
+        deserialized = load_column_profile(json.loads(serialized), config)
+        assert deserialized.data_labeler == new_mock_data_labeler
+        mock_BaseDataLabeler.assert_not_called()
+        mock_utils_DataLabeler.assert_not_called()
+
+        # validate raises error when cannot properly load data.
+        with self.assertRaisesRegex(
+            NotImplementedError,
+            "Models intialized from disk have not yet been made deserializable",
+        ):
+            class_as_dict = json.loads(serialized)
+            class_as_dict["data"]["data_labeler"] = {"from_disk": "test"}
+            deserialized = load_column_profile(class_as_dict, config)
+
+    @mock.patch(
+        "dataprofiler.profilers.profiler_utils.DataLabeler", spec=BaseDataLabeler
+    )
+    def test_json_decode_after_update(
+        self, mock_utils_DataLabeler, mock_BaseDataLabeler
+    ):
+        self._setup_data_labeler_mock(mock_BaseDataLabeler)
+        mock_utils_DataLabeler.load_from_library.side_effect = mock_BaseDataLabeler
+
+        data = pd.Series(["1", "2", "3", "4"], dtype=object)
+        expected = DataLabelerColumn(data.name)
+        expected.data_labeler._default_model_loc = "structured_model"
+        with test_utils.mock_timeit():
+            expected.update(data)
+
+        serialized = json.dumps(expected, cls=ProfileEncoder)
+        deserialized = load_column_profile(json.loads(serialized))
+
+        test_utils.assert_profiles_equal(deserialized, expected)
+        update_data = pd.Series(["4", "5", "6", "7"], dtype=object)
+        deserialized.update(update_data)
+
+        assert deserialized.sample_size == 8
+        self.assertDictEqual({"a": 4, "b": 4}, deserialized.rank_distribution)
+        np.testing.assert_array_equal(
+            np.array([4.0, 4.0]), deserialized.sum_predictions
+        )
