@@ -9,8 +9,12 @@ import numpy as np
 import pandas as pd
 
 from dataprofiler.profilers import IntColumn
+from dataprofiler.profilers.json_decoder import load_column_profile
+from dataprofiler.profilers.json_encoder import ProfileEncoder
 from dataprofiler.profilers.profiler_options import IntOptions
 
+from . import utils as test_utils
+
 test_root_path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 
 
@@ -32,10 +36,8 @@ class TestIntColumn(unittest.TestCase):
         self.assertTrue(profiler.kurtosis is np.nan)
         self.assertTrue(profiler.stddev is np.nan)
         self.assertIsNone(profiler.histogram_selection)
-        self.assertDictEqual(
-            {k: profiler.quantiles.get(k, "fail") for k in (0, 1, 2)},
-            {0: None, 1: None, 2: None},
-        )
+        self.assertIsNone(profiler.quantiles)
+        self.assertEqual(profiler._num_quantiles, 1000)
         self.assertIsNone(profiler.data_type_ratio)
 
     def test_single_data_variance_case(self):
@@ -690,6 +692,8 @@ class TestIntColumn(unittest.TestCase):
             self.assertCountEqual(expected, profiler.profile["times"])
 
     def test_profile_merge(self):
+        # Floats are not included intentionally for the test
+        # below as this is an int column
         data = [2.0, 12.5, "not an int", 6.0, "not an int"]
         df = pd.Series(data).apply(str)
         profiler1 = IntColumn("Int")
@@ -1058,8 +1062,11 @@ class TestIntColumn(unittest.TestCase):
             "median_absolute_deviation": -5,
             "t-test": {
                 "t-statistic": -0.5638091828819275,
-                "conservative": {"df": 1.0, "p-value": 0.6731699660830497},
-                "welch": {"df": 1.0547717074524683, "p-value": 0.6691886269547123},
+                "conservative": {"deg_of_free": 1.0, "p-value": 0.6731699660830497},
+                "welch": {
+                    "deg_of_free": 1.0547717074524683,
+                    "p-value": 0.6691886269547123,
+                },
             },
             "psi": 0.0675775180180274,
         }
@@ -1094,3 +1101,246 @@ class TestIntColumn(unittest.TestCase):
             str(exc.exception),
             "Unsupported operand type(s) for diff: 'IntColumn' and" " 'str'",
         )
+
+    def test_json_encode(self):
+        profiler = IntColumn("0")
+
+        serialized = json.dumps(profiler, cls=ProfileEncoder)
+
+        # Copy of NumericalStatsMixin code to test serialization of dicts
+        expected_histogram_bin_method_names = [
+            "auto",
+            "fd",
+            "doane",
+            "scott",
+            "rice",
+            "sturges",
+            "sqrt",
+        ]
+        expected_min_histogram_bin = 1000
+        expected_historam_methods = {}
+        for method in expected_histogram_bin_method_names:
+            expected_historam_methods[method] = {
+                "total_loss": 0.0,
+                "current_loss": 0.0,
+                "suggested_bin_count": expected_min_histogram_bin,
+                "histogram": {"bin_counts": None, "bin_edges": None},
+            }
+
+        serialized = json.dumps(profiler, cls=ProfileEncoder)
+        expected = json.dumps(
+            {
+                "class": "IntColumn",
+                "data": {
+                    "min": None,
+                    "max": None,
+                    "_top_k_modes": 5,
+                    "sum": 0.0,
+                    "_biased_variance": np.nan,
+                    "_biased_skewness": np.nan,
+                    "_biased_kurtosis": np.nan,
+                    "_median_is_enabled": True,
+                    "_median_abs_dev_is_enabled": True,
+                    "max_histogram_bin": 100000,
+                    "min_histogram_bin": expected_min_histogram_bin,
+                    "histogram_bin_method_names": expected_histogram_bin_method_names,
+                    "histogram_selection": None,
+                    "user_set_histogram_bin": None,
+                    "bias_correction": True,
+                    "_mode_is_enabled": True,
+                    "num_zeros": 0,
+                    "num_negatives": 0,
+                    "_num_quantiles": 1000,
+                    "histogram_methods": expected_historam_methods,
+                    "_stored_histogram": {
+                        "total_loss": 0.0,
+                        "current_loss": 0.0,
+                        "suggested_bin_count": 1000,
+                        "histogram": {"bin_counts": None, "bin_edges": None},
+                    },
+                    "_batch_history": [],
+                    "quantiles": None,
+                    "_NumericStatsMixin__calculations": {
+                        "min": "_get_min",
+                        "max": "_get_max",
+                        "sum": "_get_sum",
+                        "variance": "_get_variance",
+                        "skewness": "_get_skewness",
+                        "kurtosis": "_get_kurtosis",
+                        "histogram_and_quantiles": "_get_histogram_and_quantiles",
+                        "num_zeros": "_get_num_zeros",
+                        "num_negatives": "_get_num_negatives",
+                    },
+                    "name": "0",
+                    "col_index": np.nan,
+                    "sample_size": 0,
+                    "metadata": dict(),
+                    "times": defaultdict(),
+                    "thread_safe": True,
+                    "match_count": 0,
+                    "_IntColumn__calculations": {},
+                },
+            }
+        )
+
+        self.assertEqual(serialized, expected)
+
+    @mock.patch("time.time", return_value=0.0)
+    def test_json_encode_after_update(self, time):
+        data = np.array([0, 5, 10])
+        df = pd.Series(data).apply(str)
+
+        int_options = IntOptions()
+        int_options.histogram_and_quantiles.bin_count_or_method = 5
+        profiler = IntColumn("0", int_options)
+
+        mocked_quantiles = [0.25, 0.50, 0.75]
+        with mock.patch.object(
+            profiler, "_get_percentile", return_value=mocked_quantiles
+        ):
+            # Mock out complex _get_percentile function.
+            # Only need to test valid serialization of np.ndarry.
+            profiler.update(df)
+
+        serialized = json.dumps(profiler, cls=ProfileEncoder)
+
+        expected = json.dumps(
+            {
+                "class": "IntColumn",
+                "data": {
+                    "min": 0.0,
+                    "max": 10.0,
+                    "_top_k_modes": 5,
+                    "sum": 15.0,
+                    "_biased_variance": 16.666666666666668,
+                    "_biased_skewness": 0.0,
+                    "_biased_kurtosis": -1.5,
+                    "_median_is_enabled": True,
+                    "_median_abs_dev_is_enabled": True,
+                    "max_histogram_bin": 100000,
+                    "min_histogram_bin": 1000,
+                    "histogram_bin_method_names": ["custom"],
+                    "histogram_selection": None,
+                    "user_set_histogram_bin": None,
+                    "bias_correction": True,
+                    "_mode_is_enabled": True,
+                    "num_zeros": 1,
+                    "num_negatives": 0,
+                    "histogram_selection": None,
+                    "user_set_histogram_bin": 5,
+                    "bias_correction": True,
+                    "_mode_is_enabled": True,
+                    "num_zeros": 1,
+                    "num_negatives": 0,
+                    "_num_quantiles": 1000,
+                    "histogram_methods": {
+                        "custom": {
+                            "total_loss": 0.0,
+                            "current_loss": 0.0,
+                            "suggested_bin_count": 5,
+                            "histogram": {"bin_counts": None, "bin_edges": None},
+                        }
+                    },
+                    "_stored_histogram": {
+                        "total_loss": 2.0,
+                        "current_loss": 2.0,
+                        "suggested_bin_count": 1000,
+                        "histogram": {
+                            "bin_counts": [1, 0, 1, 0, 1],
+                            "bin_edges": [0.0, 2.0, 4.0, 6.0, 8.0, 10.0],
+                        },
+                    },
+                    "_batch_history": [
+                        {
+                            "match_count": 3,
+                            "sample_size": 3,
+                            "min": 0.0,
+                            "max": 10.0,
+                            "sum": 15.0,
+                            "biased_variance": 16.666666666666668,
+                            "mean": 5.0,
+                            "biased_skewness": 0.0,
+                            "biased_kurtosis": -1.5,
+                            "num_zeros": 1,
+                            "num_negatives": 0,
+                        }
+                    ],
+                    "quantiles": mocked_quantiles,
+                    "_NumericStatsMixin__calculations": {
+                        "min": "_get_min",
+                        "max": "_get_max",
+                        "sum": "_get_sum",
+                        "variance": "_get_variance",
+                        "skewness": "_get_skewness",
+                        "kurtosis": "_get_kurtosis",
+                        "histogram_and_quantiles": "_get_histogram_and_quantiles",
+                        "num_zeros": "_get_num_zeros",
+                        "num_negatives": "_get_num_negatives",
+                    },
+                    "name": "0",
+                    "col_index": np.nan,
+                    "sample_size": 3,
+                    "metadata": dict(),
+                    "times": {
+                        "min": 0.0,
+                        "max": 0.0,
+                        "sum": 0.0,
+                        "variance": 0.0,
+                        "skewness": 0.0,
+                        "kurtosis": 0.0,
+                        "histogram_and_quantiles": 0.0,
+                        "num_zeros": 0.0,
+                        "num_negatives": 0.0,
+                    },
+                    "thread_safe": True,
+                    "match_count": 3,
+                    "_IntColumn__calculations": {},
+                },
+            }
+        )
+
+        self.assertEqual(serialized, expected)
+
+    def test_json_decode(self):
+        fake_profile_name = None
+        expected_profile = IntColumn(fake_profile_name)
+
+        serialized = json.dumps(expected_profile, cls=ProfileEncoder)
+        deserialized = load_column_profile(json.loads(serialized))
+
+        test_utils.assert_profiles_equal(deserialized, expected_profile)
+
+    def test_json_decode_after_update(self):
+        fake_profile_name = "Fake profile name"
+        # Actual deserialization
+
+        # Build expected IntColumn
+        df_int = pd.Series([-1, 2, 5, 7, 4, 3, 2, 0, 0, 9])
+        expected_profile = IntColumn(fake_profile_name)
+
+        with test_utils.mock_timeit():
+            expected_profile.update(df_int)
+
+        # Validate reporting before deserialization
+        expected_profile.report()
+
+        serialized = json.dumps(expected_profile, cls=ProfileEncoder)
+        deserialized = load_column_profile(json.loads(serialized))
+
+        # Validate reporting after deserialization
+        deserialized.report()
+        test_utils.assert_profiles_equal(deserialized, expected_profile)
+
+        df_int = pd.Series(
+            [
+                4,  # add existing
+                15,  # add new
+            ]
+        )
+
+        # validating update after deserialization
+        deserialized.update(df_int)
+
+        assert deserialized.sample_size == 12
+        assert deserialized.mean == sum([-1, 2, 5, 7, 4, 3, 2, 0, 0, 9, 4, 15]) / 12
+        assert deserialized.max == 15
