@@ -1,3 +1,4 @@
+import json
 import os
 import unittest
 import warnings
@@ -7,7 +8,9 @@ from unittest import mock
 import numpy as np
 import pandas as pd
 
-from dataprofiler.profilers import TextColumn, utils
+from dataprofiler.profilers import TextColumn, profiler_utils
+from dataprofiler.profilers.json_decoder import load_column_profile
+from dataprofiler.profilers.json_encoder import ProfileEncoder
 from dataprofiler.profilers.profiler_options import TextOptions
 from dataprofiler.tests.profilers import utils as test_utils
 
@@ -581,13 +584,16 @@ class TestTextColumnProfiler(unittest.TestCase):
             "median_absolute_deviation": -0.5,
             "variance": profile1["variance"] - profile2["variance"],
             "stddev": profile1["stddev"] - profiler2["stddev"],
-            "vocab": utils.find_diff_of_lists_and_sets(
+            "vocab": profiler_utils.find_diff_of_lists_and_sets(
                 profile1["vocab"], profile2["vocab"]
             ),
             "t-test": {
                 "t-statistic": -1.9339958714826413,
-                "conservative": {"df": 8.0, "p-value": 0.08916903961929257},
-                "welch": {"df": 15.761400272034564, "p-value": 0.07127621949432528},
+                "conservative": {"deg_of_free": 8.0, "p-value": 0.08916903961929257},
+                "welch": {
+                    "deg_of_free": 15.761400272034564,
+                    "p-value": 0.07127621949432528,
+                },
             },
         }
 
@@ -607,3 +613,210 @@ class TestTextColumnProfiler(unittest.TestCase):
             places=2,
         )
         self.assertDictEqual(expected_diff, profile_diff)
+
+    @mock.patch("time.time", return_value=0.0)
+    def test_json_encode_after_update(self, time):
+        df = pd.Series(
+            [
+                "abcd",
+                "aa",
+                "abcd",
+                "lito-potamus",
+                "b",
+                "4",
+                ".098",
+                "2",
+                "dfd",
+                "2",
+                "12.32",
+            ]
+        ).apply(str)
+
+        text_options = TextOptions()
+        text_options.histogram_and_quantiles.bin_count_or_method = 5
+        profiler = TextColumn(df.name, text_options)
+        with test_utils.mock_timeit():
+            profiler.update(df)
+
+        serialized_dict = json.loads(json.dumps(profiler, cls=ProfileEncoder))
+
+        # popping quantiles and comparing as list below since it is so large
+        serialized_quantiles = serialized_dict["data"].pop("quantiles")
+
+        # popping vocab and comparing as set below since order is random
+        serialized_vocab = serialized_dict["data"].pop("vocab")
+
+        serialized = json.dumps(serialized_dict)
+
+        expected = json.dumps(
+            {
+                "class": "TextColumn",
+                "data": {
+                    "min": 1.0,
+                    "max": 12.0,
+                    "_top_k_modes": 5,
+                    "sum": 38.0,
+                    "_biased_variance": 9.33884297520661,
+                    "_biased_skewness": 1.8025833203700588,
+                    "_biased_kurtosis": 2.7208317017777395,
+                    "_median_is_enabled": True,
+                    "_median_abs_dev_is_enabled": True,
+                    "max_histogram_bin": 100000,
+                    "min_histogram_bin": 1000,
+                    "histogram_bin_method_names": ["custom"],
+                    "histogram_selection": None,
+                    "user_set_histogram_bin": 5,
+                    "bias_correction": True,
+                    "_mode_is_enabled": True,
+                    "num_zeros": 0,
+                    "num_negatives": 0,
+                    "_num_quantiles": 1000,
+                    "histogram_methods": {
+                        "custom": {
+                            "total_loss": 0.0,
+                            "current_loss": 0.0,
+                            "suggested_bin_count": 5,
+                            "histogram": {"bin_counts": None, "bin_edges": None},
+                        }
+                    },
+                    "_stored_histogram": {
+                        "total_loss": 7.63,
+                        "current_loss": 7.63,
+                        "suggested_bin_count": 1000,
+                        "histogram": {
+                            "bin_counts": [6, 4, 0, 0, 1],
+                            "bin_edges": [1.0, 3.2, 5.4, 7.6000000000000005, 9.8, 12.0],
+                        },
+                    },
+                    "_batch_history": [
+                        {
+                            "match_count": 11,
+                            "sample_size": 11,
+                            "min": 1.0,
+                            "max": 12.0,
+                            "sum": 38.0,
+                            "biased_variance": 9.33884297520661,
+                            "mean": 3.4545454545454546,
+                            "biased_skewness": 1.8025833203700588,
+                            "biased_kurtosis": 2.7208317017777395,
+                        }
+                    ],
+                    "_NumericStatsMixin__calculations": {
+                        "min": "_get_min",
+                        "max": "_get_max",
+                        "sum": "_get_sum",
+                        "variance": "_get_variance",
+                        "skewness": "_get_skewness",
+                        "kurtosis": "_get_kurtosis",
+                        "histogram_and_quantiles": "_get_histogram_and_quantiles",
+                    },
+                    "name": None,
+                    "col_index": np.nan,
+                    "sample_size": 11,
+                    "metadata": {},
+                    "times": {
+                        "vocab": 1.0,
+                        "min": 1.0,
+                        "max": 1.0,
+                        "sum": 1.0,
+                        "variance": 1.0,
+                        "skewness": 1.0,
+                        "kurtosis": 1.0,
+                        "histogram_and_quantiles": 1.0,
+                    },
+                    "thread_safe": True,
+                    "match_count": 11,
+                    "_TextColumn__calculations": {"vocab": "_update_vocab"},
+                    "type": "string",
+                },
+            }
+        )
+
+        expected_vocab = profiler.vocab
+        expected_quantiles = profiler.quantiles
+
+        self.assertEqual(serialized, expected)
+        self.assertSetEqual(set(serialized_vocab), set(expected_vocab))
+        self.assertListEqual(serialized_quantiles, expected_quantiles)
+
+    def test_json_decode(self):
+        fake_profile_name = None
+        expected_profile = TextColumn(fake_profile_name)
+
+        serialized = json.dumps(expected_profile, cls=ProfileEncoder)
+        deserialized = load_column_profile(json.loads(serialized))
+
+        test_utils.assert_profiles_equal(deserialized, expected_profile)
+
+    def test_json_decode_after_update(self):
+        fake_profile_name = "Fake profile name"
+        # Actual deserialization
+
+        # Build expected IntColumn
+        df_int = pd.Series(
+            [
+                "abcd",
+                "aa",
+                "abcd",
+                "lito-potamus",
+                "b",
+                "4",
+                ".098",
+                "2",
+                "dfd",
+                "2",
+                "12.32",
+            ]
+        )
+        expected_profile = TextColumn(fake_profile_name)
+
+        with test_utils.mock_timeit():
+            expected_profile.update(df_int)
+
+        # Validate reporting before deserialization
+        expected_profile.report()
+
+        serialized = json.dumps(expected_profile, cls=ProfileEncoder)
+        deserialized = load_column_profile(json.loads(serialized))
+
+        # Validate reporting after deserialization
+        deserialized.report()
+        test_utils.assert_profiles_equal(deserialized, expected_profile)
+
+        df_str = pd.Series(
+            [
+                "aa",  # add existing
+                "awsome",  # add new
+            ]
+        )
+
+        # validating update after deserialization
+        deserialized.update(df_str)
+
+        assert deserialized.sample_size == 13
+        assert set(deserialized.vocab) == {
+            ".",
+            "-",
+            "1",
+            "2",
+            "3",
+            "4",
+            "8",
+            "9",
+            "0",
+            "a",
+            "b",
+            "c",
+            "d",
+            "e",
+            "f",
+            "i",
+            "l",
+            "m",
+            "o",
+            "p",
+            "s",
+            "t",
+            "u",
+            "w",
+        }
