@@ -1,3 +1,4 @@
+import numbers
 import os
 import random
 import shutil
@@ -6,6 +7,12 @@ from unittest import mock
 import numpy as np
 
 import dataprofiler as dp
+from dataprofiler.profilers.base_column_profilers import BaseColumnProfiler
+from dataprofiler.profilers.column_profile_compilers import BaseCompiler
+from dataprofiler.profilers.profile_builder import BaseProfiler, StructuredColProfiler
+from dataprofiler.profilers.profiler_options import BaseOption
+from dataprofiler.profilers.profiler_utils import find_diff_of_dicts
+from dataprofiler.tests.test_utils import patched_assert_warns
 
 
 def set_seed(seed=None):
@@ -161,3 +168,58 @@ def mock_timeit(*args, **kwargs):
 
     counter = increment_counter()
     return mock.patch("time.time", side_effect=lambda: next(counter))
+
+
+def assert_profiles_equal(actual, expected):
+    """
+    Checks if two profile objects are equal.
+
+    profiles are instances of BaseProfiler or BaseColumnProfiler. Throws
+        exception if not equal
+
+    :param actual: profile to compare to expected
+    :type actual: instance of BaseProfiler or BaseColumnProfiler
+    :param expected: profile to compare to actual
+    :type expected: instance of BaseProfiler or BaseColumnProfiler
+    """
+    actual_dict = actual.__dict__ if not isinstance(actual, dict) else actual
+    expected_dict = expected.__dict__ if not isinstance(expected, dict) else expected
+
+    assert (
+        actual_dict.keys() == expected_dict.keys()
+    ), f"{actual_dict.keys()} != {expected_dict.keys()}"
+
+    for key in expected_dict.keys():
+        actual_value = actual_dict.get(key, None)
+        expected_value = expected_dict.get(key, None)
+
+        assert type(actual_value) == type(
+            expected_value
+        ), f"{actual_value} with type {type(actual_value)} and \
+            {expected_value} with type {type(expected_value)} \
+            do not have the same type for key: {key}"
+
+        if key == "_profile" and isinstance(actual_value, list):
+            for x in range(len(actual_value)):
+                assert_profiles_equal(actual_value[x], expected_value[x])
+        elif isinstance(
+            actual_value,
+            (
+                BaseProfiler,
+                BaseColumnProfiler,
+                StructuredColProfiler,
+                BaseCompiler,
+                BaseOption,
+            ),
+        ):
+            assert_profiles_equal(actual_value, expected_value)
+        elif isinstance(actual_value, dict):
+            assert_profiles_equal(actual_value, expected_value)
+        elif isinstance(actual_value, numbers.Number):
+            np.testing.assert_equal(actual_value, expected_value, f"{key}")
+        elif isinstance(actual_value, np.ndarray):
+            np.testing.assert_array_equal(actual_value, expected_value)
+        else:
+            assert (
+                actual_value == expected_value
+            ), f"Actual value of {actual_value} and expected value of {expected_value} do not have the same value for key: {key}"
